@@ -1,42 +1,43 @@
-;;;  mh-e.el	(Version: 3.7 for GNU Emacs Version 18 and MH.5 and MH.6)
+;;; mh-e.el --- GNU Emacs interface to the MH mailer
+
+;;;  Copyright (C) 1985, 86, 87, 88, 89, 92 Free Software Foundation
+
+;; Author: James Larus <larus@ginger.berkeley.edu>
+;; Version: 3.7
+;; Keywords: mail
 
 (defvar mh-e-RCS-id)
-(setq mh-e-RCS-id "$Header: /var/home/larus/lib/emacs/RCS/mh-e.el,v 3.1 90/09/28 15:47:58 larus Exp Locker: larus $")
-(provide 'mh-e)
-
-;;;  Copyright (C) 1985-89 Free Software Foundation
-;;;     Author:  James Larus (larus@ginger.Berkeley.EDU or ucbvax!larus)
-;;;	Please send suggestions and corrections to the above address.
-;;;
-;;;  This file contains mh-e, a GNU Emacs front end to the MH mail system.
-
-;; This file is part of GNU Emacs.
-
-;; GNU Emacs is free software; you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 1, or (at your option)
-;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
+;; but without any warranty.  No author or distributor
+;; accepts responsibility to anyone for the consequences of using it
+;; or for whether it serves any particular purpose or works at all,
+;; unless he says so in writing.
 
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; Everyone is granted permission to copy, modify and redistribute
+;; GNU Emacs, but only under the conditions described in the
+;; document "GNU Emacs copying permission notice".   An exact copy
+;; of the document is supposed to have been given to you along with
+;; GNU Emacs so that you can know how you may redistribute it all.
+;; It should be in a file named COPYING.  Among other things, the
+;; copyright notice and this notice must be preserved on all copies.
 
+;;; Commentary:
+
+;;;  This file contains mh-e, a GNU Emacs front end to the MH mail system
+;;;  (specifically, for use with MH.5 and MH.6).
 
 ;;;  Original version for Gosling emacs by Brian Reid, Stanford, 1982.
 ;;;  Modified by James Larus, BBN, July 1984 and UCB, 1984 & 1985.
 ;;;  Rewritten for GNU Emacs, James Larus 1985.  larus@ginger.berkeley.edu
 ;;;  Modified by Stephen Gildea 1988.  gildea@bbn.com
 
-
 ;;;  NB.  MH must have been compiled with the MHE compiler flag or several
 ;;;  features necessary mh-e will be missing from MH commands, specifically
 ;;;  the -build switch to repl and forw.
 
+;;; Code:
+
 
 
 ;;; Constants:
@@ -54,13 +55,13 @@ This is the case when `send' is compiled with the BERK option.")
 ;;; Hooks:
 
 (defvar mh-folder-mode-hook nil
-  "Invoked in mh-folder-mode on a new folder.")
+  "Invoked in `mh-folder mode' on a new folder.")
 
 (defvar mh-letter-mode-hook nil
-  "Invoked in mh-letter-mode on a new letter.")
+  "Invoked in `mh-letter-mode' on a new letter.")
 
 (defvar mh-compose-letter-function nil
-  "Invoked in mh-compose-and-send-mail on a draft letter.
+  "Invoked in `mh-compose-and-send-mail' on a draft letter.
 It is passed three arguments: TO recipients, SUBJECT, and CC recipients.")
 
 (defvar mh-before-send-letter-hook nil
@@ -74,7 +75,7 @@ It is passed three arguments: TO recipients, SUBJECT, and CC recipients.")
 
 
 (defvar mh-ins-string nil
-  "Temporarily set by mh-insert-prefix prior to running mh-yank-hooks.")
+  "Temporarily set by `mh-insert-prefix' prior to running `mh-yank-hooks'.")
 
 (defvar mh-yank-hooks
   '(lambda ()
@@ -84,21 +85,21 @@ It is passed three arguments: TO recipients, SUBJECT, and CC recipients.")
       (while (< (point) (mark))
 	(insert mh-ins-string)
 	(forward-line 1))))
-  "Hook to run citation function.  Expects POINT and MARK to be set to
-the region to cite.")
+  "Hook to run citation function.
+Expects POINT and MARK to be set to the region to cite.")
 
 
 ;;; Personal preferences:
 
 (defvar mh-clean-message-header nil
   "*Non-nil means clean headers of messages that are displayed or inserted.
-The variables mh-visible-headers and mh-invisible-headers control what is
-removed.")
+The variables `mh-visible-headers' and `mh-invisible-headers' control what
+is removed.")
 
 (defvar mh-visible-headers nil
   "*If non-nil, contains a regexp specifying the headers to keep when cleaning.
-Only used if mh-clean-message-header is non-nil.  Setting this variable
-overrides mh-invisible-headers.")
+Only used if `mh-clean-message-header' is non-nil.  Setting this variable
+overrides `mh-invisible-headers'.")
 
 (defvar mhl-formfile nil
   "*Name of format file to be used by mhl to show messages.
@@ -197,7 +198,7 @@ Matches lines marked as deleted, refiled, in a sequence, or the cur message.")
   "Regexp matching scan line for the cur message.")
 
 (defvar mh-show-buffer-mode-line-buffer-id "{%%b}  %s/%d"
-  "Format string to produce mode-line-buffer-id for show buffers.
+  "Format string to produce `mode-line-buffer-id' for show buffers.
 First argument is folder name.  Second is message number.")
 
 (defvar mh-partial-folder-mode-line-annotation "select"
@@ -210,7 +211,8 @@ The string is displayed after the folder's name.  NIL for no annotation.")
 (defvar mh-invisible-headers
   "^Received: \\|^Message-Id: \\|^Remailed-\\|^Via: \\|^Mail-from: \\|^Return-Path: \\|^In-Reply-To: \\|^Resent-"
   "Regexp matching lines in a message header that are not to be shown.
-If mh-visible-headers is non-nil, it is used instead to specify what to keep.")
+If `mh-visible-headers' is non-nil, it is used instead to specify what
+to keep.")
 
 (defvar mh-rejected-letter-start "^   ----- Unsent message follows -----$"
   "Regexp specifying the beginning of the wrapper around a returned letter.
@@ -264,6 +266,9 @@ NIL means do not use draft folder.")
 (defvar mh-previous-seq nil
   "Name of the sequence to which a message was last added.")
 
+(defvar mh-signature-file-name "~/.signature"
+  "Name of file containing the user's signature.")
+
 
 ;;; Macros and generic functions:
 
@@ -301,6 +306,7 @@ NIL means do not use draft folder.")
 
 ;;; Entry points:
 
+;;;###autoload
 (defun mh-rmail (&optional arg)
   "Inc(orporate) new mail (no arg) or scan a MH mail box (arg given).
 This front end uses the MH mail system, which uses different conventions
@@ -312,6 +318,7 @@ from the usual mail system."
       (mh-inc-folder)))
 
 
+;;;###autoload
 (defun mh-smail ()
   "Compose and send mail with the MH mail system."
   (interactive)
@@ -345,7 +352,7 @@ digest are inserted into the folder after that message."
 
 (defun mh-copy-msg (prefix-provided msg-or-seq dest)
   "Copy specified MESSAGE(s) to another FOLDER without deleting them.
-Default is the displayed message.  If (optional) prefix argument is
+Default is the displayed message.  If optional prefix argument is
 provided, then prompt for the message sequence."
   (interactive (list current-prefix-arg
 		     (if current-prefix-arg
@@ -360,8 +367,8 @@ provided, then prompt for the message sequence."
 
 (defun mh-delete-msg (msg-or-seq)
   "Mark the specified MESSAGE(s) for subsequent deletion and move to the next.
-Default is the displayed message.  If (optional) prefix argument is
-provided, then prompt for the message sequence."
+Default is the displayed message.  If optional prefix argument is
+given then prompt for the message sequence."
   (interactive (list (if current-prefix-arg
 			 (mh-read-seq-default "Delete" t)
 			 (mh-get-msg-num t))))
@@ -373,7 +380,7 @@ provided, then prompt for the message sequence."
 
 (defun mh-delete-msg-no-motion (msg-or-seq)
   "Mark the specified MESSAGE(s) for subsequent deletion.
-Default is the displayed message.  If (optional) prefix argument is
+Default is the displayed message.  If optional prefix argument is
 provided, then prompt for the message sequence."
   (interactive (list (if current-prefix-arg
 			 (mh-read-seq-default "Delete" t)
@@ -385,8 +392,8 @@ provided, then prompt for the message sequence."
 
 (defun mh-delete-msg-from-seq (prefix-provided msg-or-seq &optional from-seq)
   "Delete MESSAGE (default: displayed message) from SEQUENCE.
-If (optional) prefix argument provided, then delete all messages from a
-sequence."
+If optional prefix argument provided, then delete all messages
+from a sequence."
   (interactive (let ((argp current-prefix-arg))
 		 (list argp
 		       (if argp
@@ -462,7 +469,7 @@ Default is the displayed message."
 
 (defun mh-forward (prefix-provided msg-or-seq to cc)
   "Forward MESSAGE(s) (default: displayed message).
-If (optional) prefix argument provided, then prompt for the message sequence."
+If optional prefix argument provided, then prompt for the message sequence."
   (interactive (list current-prefix-arg
 		     (if current-prefix-arg
 			 (mh-read-seq-default "Forward" t)
@@ -538,7 +545,7 @@ Return non-nil if cursor is at message."
   "Inc(orporate) new mail into +inbox.
 Optional prefix argument specifies an alternate maildrop from the default.
 If this is given, mail is incorporated into the current folder, rather
-than +inbox.  Run mh-inc-folder-hook after incorporating new mail."
+than +inbox.  Run `mh-inc-folder-hook' after incorporating new mail."
   (interactive (list (if current-prefix-arg
 			 (expand-file-name
 			  (read-file-name "inc mail from file: "
@@ -635,7 +642,7 @@ Reads which sequence.  Use \\[mh-widen] to undo this command."
 (defun mh-pack-folder (range)
   "Renumber the messages of a folder to be 1..n.
 First, offer to execute any outstanding commands for the current folder.
-If (optional) prefix argument provided, prompt for the range of messages
+If optional prefix argument provided, prompt for the range of messages
 to display after packing.  Otherwise, show the entire folder."
   (interactive (list (if current-prefix-arg
 			 (mh-read-msg-range
@@ -648,7 +655,7 @@ to display after packing.  Otherwise, show the entire folder."
 
 (defun mh-pipe-msg (prefix-provided command)
   "Pipe the current message through the given shell COMMAND.
-If (optional) prefix argument is provided, send the entire message.
+If optional prefix argument is provided, send the entire message.
 Otherwise just send the message's body."
   (interactive
    (list current-prefix-arg (read-string "Shell command on message: ")))
@@ -661,7 +668,7 @@ Otherwise just send the message's body."
 
 (defun mh-refile-msg (prefix-provided msg-or-seq dest)
   "Refile MESSAGE(s) (default: displayed message) in FOLDER.
-If (optional) prefix argument provided, then prompt for message sequence."
+If optional prefix argument provided, then prompt for message sequence."
   (interactive
    (list current-prefix-arg
 	 (if current-prefix-arg
@@ -698,7 +705,7 @@ previous refile or write command."
 
 (defun mh-reply (prefix-provided msg)
   "Reply to a MESSAGE (default: displayed message).
-If (optional) prefix argument provided, then include the message in the reply
+If optional prefix argument provided, then include the message in the reply
 using filter mhl.reply in your MH directory."
   (interactive (list current-prefix-arg (mh-get-msg-num t)))
   (let ((minibuffer-help-form
@@ -837,7 +844,7 @@ Scrolls ARG lines or a full screen if no argument is supplied."
 
 (defun mh-print-msg (prefix-provided msg-or-seq)
   "Print MESSAGE(s) (default: displayed message) on a line printer.
-If (optional) prefix argument provided, then prompt for the message sequence."
+If optional prefix argument provided, then prompt for the message sequence."
   (interactive (list current-prefix-arg
 		     (if current-prefix-arg
 			 (reverse (mh-seq-to-msgs
@@ -885,7 +892,7 @@ If (optional) prefix argument provided, then prompt for the message sequence."
 
 (defun mh-put-msg-in-seq (prefix-provided from to)
   "Add MESSAGE(s) (default: displayed message) to SEQUENCE.
-If (optional) prefix argument provided, then prompt for the message sequence."
+If optional prefix argument provided, then prompt for the message sequence."
   (interactive (list current-prefix-arg
 		     (if current-prefix-arg
 			 (mh-seq-to-msgs
@@ -898,7 +905,7 @@ If (optional) prefix argument provided, then prompt for the message sequence."
 
 (defun mh-rescan-folder (range)
   "Rescan a folder after optionally processing the outstanding commands.
-If (optional) prefix argument is provided, prompt for the range of
+If optional prefix argument is provided, prompt for the range of
 messages to display.  Otherwise show the entire folder."
   (interactive (list (if current-prefix-arg
 			 (mh-read-msg-range "Range to scan [all]? ")
@@ -1062,7 +1069,7 @@ mh-summary-height) and the show buffer below it."
 
 (defun mh-undo (prefix-provided msg-or-seq)
   "Undo the deletion or refile of the specified MESSAGE(s).
-Default is the displayed message.  If (optional) prefix argument is
+Default is the displayed message.  If optional prefix argument is
 provided, then prompt for the message sequence."
   (interactive (list current-prefix-arg
 		     (if current-prefix-arg
@@ -1762,9 +1769,9 @@ invoked with no args, if those values are non-nil.
   (mh-set-mode-name "mh-e letter")
   (set-syntax-table mh-letter-mode-syntax-table)
   (run-hooks 'text-mode-hook 'mh-letter-mode-hook)
-  (mh-when auto-fill-hook
-    (make-local-variable 'auto-fill-hook)
-    (setq auto-fill-hook 'mh-auto-fill-for-letter)))
+  (mh-when auto-fill-function
+    (make-local-variable 'auto-fill-function)
+    (setq auto-fill-function 'mh-auto-fill-for-letter)))
 
 
 (defun mh-auto-fill-for-letter ()
@@ -1781,7 +1788,7 @@ invoked with no args, if those values are non-nil.
   ;; Return non-nil if the point is in the header of a draft message.
   (save-excursion
     (let ((cur-point (point)))
-      (goto-char (dot-min))
+      (goto-char (point-min))
       (re-search-forward "^--------" nil t)
       (< cur-point (point)))))
 
@@ -1804,7 +1811,7 @@ it does not exist.  Set the mark to point before moving."
 	       (insert ", ")
 	       (insert " ")))
 	  (t
-	   (goto-char (dot-min))
+	   (goto-char (point-min))
 	   (re-search-forward "^To:")
 	   (forward-line 1)
 	   (while (looking-at "^[ \t]") (forward-line 1))
@@ -1827,7 +1834,7 @@ Prompt for the field name with a completion list of the current folders."
 (defun mh-insert-signature ()
   "Insert the file ~/.signature at the current point."
   (interactive "")
-  (insert-file-contents "~/.signature")
+  (insert-file-contents mh-signature-file-name)
   (set-buffer-modified-p (buffer-modified-p))) ; force mode line update
 
 
@@ -1972,7 +1979,7 @@ Put messages found in a sequence named `search'."
 
 (defun mh-send-letter (&optional arg)
   "Send the draft letter in the current buffer.
-If (optional) prefix argument is provided, monitor delivery.
+If optional prefix argument is provided, monitor delivery.
 Run mh-before-send-letter-hook before doing anything."
   (interactive "P")
   (run-hooks 'mh-before-send-letter-hook)
@@ -2021,9 +2028,8 @@ Run mh-before-send-letter-hook before doing anything."
   "Insert a message from any folder into the current letter.
 Removes the message's headers using mh-invisible-headers.
 Prefixes each non-blank line with mh-ins-buf-prefix (default \">> \").
-If (optional) prefix argument provided, do not indent and do not delete
-headers.
-Leaves the mark before the letter and point after it."
+If optional prefix argument provided, do not indent and do not delete
+headers.  Leaves the mark before the letter and point after it."
   (interactive
    (list current-prefix-arg
 	 (mh-prompt-for-folder "Message from" mh-sent-from-folder nil)
@@ -2047,11 +2053,11 @@ Leaves the mark before the letter and point after it."
 (defun mh-yank-cur-msg ()
   "Insert the current message into the draft buffer.
 Prefix each non-blank line in the message with the string in
-mh-ins-buf-prefix.  If a region is set in the message's buffer, then
+`mh-ins-buf-prefix'.  If a region is set in the message's buffer, then
 only the region will be inserted.  Otherwise, the entire message will
-be inserted if mh-yank-from-start-of-msg is non-nil.  If this variable
+be inserted if `mh-yank-from-start-of-msg' is non-nil.  If this variable
 is nil, the portion of the message following the point will be yanked.
-If mh-delete-yanked-msg-window is non-nil, any window displaying the
+If `mh-delete-yanked-msg-window' is non-nil, any window displaying the
 yanked message will be deleted."
   (interactive)
   (if (and mh-sent-from-folder mh-sent-from-msg)
@@ -2638,7 +2644,7 @@ Result is in a form suitable for completing read."
 
 
 (defun mh-expand-file-name (filename &optional default)
-  "Just like expand-file-name, but also handles MH folder names.
+  "Just like `expand-file-name', but also handles MH folder names.
 Assumes that any filename that starts with '+' is a folder name."
    (if (mh-folder-name-p filename)
        (expand-file-name (substring filename 1) mh-user-path)
@@ -2734,6 +2740,52 @@ Assumes that any filename that starts with '+' is a folder name."
   (if (re-search-forward "^$\\|^-+$" nil nil)
       (forward-line arg)))
 
+
+(defun mh-unshar (dir)
+  "Unshar the current message in the directory given by DIR."
+  (interactive "DUnshar in directory: ")
+  (let ((default-directory default-directory)
+        (errbuf " *Unshar Output*")
+	(curbuf (current-buffer))
+	(show-buffer mh-show-buffer)
+	start
+	)
+    (setq dir (expand-file-name dir))
+    (if (not (eq system-type 'vax-vms))
+	(setq dir (file-name-as-directory dir)))
+    (mh-show nil)		;;; force showing of current message
+    (save-excursion
+      (set-buffer show-buffer)
+      (goto-char (point-min))
+      (message "Looking for start of shar package ...")
+      (if (or (re-search-forward "^#![ \t]*/bin/sh" nil t)
+	      (and (re-search-forward "^[^a-z0-9\"]*cut here" nil t)
+		   (forward-line 1))
+	      (re-search-forward "^#" nil t)
+	      (re-search-forward "^: " nil t)
+	      )
+	  (progn
+	    (beginning-of-line)
+	    (setq start (point))
+	    (set-buffer curbuf)
+	    (pop-to-buffer errbuf)
+	    (kill-region (point-max) (point-min))
+	    (insert (format "Unsharing in directory \"%s\" ...\n\n" dir))
+	    (message "Please wait ...")
+	    (sit-for 0)
+	    (set-buffer show-buffer)
+	    (setq default-directory dir)
+	    (call-process-region start (point-max)
+				 "/bin/sh" nil errbuf t)
+	    (pop-to-buffer curbuf)
+	    (message "Unshar done")
+	    )
+	(error "Can't find start of shar file")
+	)
+      )
+    )
+  )
+
 
 
 ;;; Build the folder-mode keymap:
@@ -2756,6 +2808,7 @@ Assumes that any filename that starts with '+' is a folder name."
 (define-key mh-folder-mode-map "\ef" 'mh-visit-folder)
 (define-key mh-folder-mode-map "\ek" 'mh-kill-folder)
 (define-key mh-folder-mode-map "\el" 'mh-list-folders)
+(define-key mh-folder-mode-map "\en" 'mh-unshar)
 (define-key mh-folder-mode-map "\eo" 'mh-write-msg-to-file)
 (define-key mh-folder-mode-map "\ep" 'mh-pack-folder)
 (define-key mh-folder-mode-map "\es" 'mh-search-folder)
@@ -2826,7 +2879,10 @@ Assumes that any filename that starts with '+' is a folder name."
 
 ;;; For Gnu Emacs.
 ;;; Local Variables: ***
-;;; eval: (put 'mh-when 'lisp-indent-hook 1) ***
-;;; eval: (put 'with-mh-folder-updating 'lisp-indent-hook 1) ***
+;;; eval: (put 'mh-when 'lisp-indent-function 1) ***
+;;; eval: (put 'with-mh-folder-updating 'lisp-indent-function 1) ***
 ;;; End: ***
 
+(provide 'mh-e)
+
+;;; mh-e.el ends here
