@@ -34,7 +34,9 @@
 (defvar Info-default-directory-list
   (list "/usr/local/info/"
 	"/usr/local/lib/info/"
-	(expand-file-name "../info/" data-directory))
+	configure-info-directory
+	(expand-file-name "../info/" data-directory)
+	(expand-file-name "../../info/" data-directory))
   "List of directories to search for Info documentation files.")
 
 (defvar news-path "/usr/spool/news/"
